using DataFrames
using XLSX
using JuMP
using Gurobi
using Gadfly

#data
slats_dim = XLSX.readdata("Dataset0.xlsx", "Slats", "A5:C146")
panels_dim = XLSX.readdata("Dataset0.xlsx", "Panels", "B4:C4")

#Sets
P = XLSX.readdata("Dataset0.xlsx", "Panels", "E4") #Max nb of panels
S = slats_dim[:,1] #Slat associated number
L = panels_dim[1] #Length of panels to create
W = panels_dim[2] #Width of panels to create
I = L - 1 #Ordinate of the last gridded area of a panel
J = W - 1 #Abscissa of the last gridded area of a panel

#Parameters
l = slats_dim[:,2] #Length of a slat
w = slats_dim[:,3] #Width of a slat

#Model definition
model = Model(optimizer_with_attributes(Gurobi.Optimizer,"TimeLimit"=>3600))

#Variables definition
#X_p,st,s : binary variable, equal to 1 if the Bottom Left Corner (BLC) of slat s is positionned on the position (i,j) of panel p
@variable(model, X[p = 1:P, s = 1:length(S), i = 0:I, j = 0:J], Bin)
#Y_p : binary variable, equal to 1 if panel p is created
@variable(model, Z[p = 1:P, s = 1:length(S), i = 0:I, j = 0:J], Bin)
#Z_p,st,s : binary variable, equal to 1 if the position (i,j) of panel p is occupied by slat s
@variable(model, Y[p = 1:P], Bin)

#Objective function
@objective(model, Max, sum(Y[p] for p = 1:P))

#Constraints
#1 A slay can only be used once
for s = 1:length(S)
        @constraint(model, sum(X[p,s,i,j] for p = 1:P for i = 0:I for j = 0:J) <= 1)
end

#2 All positions of a panel must be occupied in order to create it
for p = 1:P
    @constraint(model, sum(Z[p,s,i,j] for s = 1:length(S) for i = 0:I for j = 0:J) >= L*W*Y[p])
end

#3 A position can only be occupied by one slat (NO OVERLAP)
for p = 1:P
    for i = 0:I
        for j = 0:J
            @constraint(model, sum(Z[p,s,i,j] for s = 1:length(S)) <= 1)
        end
    end
end

#4 If a position is occupied by the BLC of a slat, the following "dimensions of the slat" positions of the panel are occupied as well
for p = 1:P
    for s = 1:length(S)
        for i = 0:I
            for j = 0:J
                @constraint(model, X[p,s,i,j]*min(l[s],L-i)*min(w[s],W-j) <= sum(Z[p,s,k,l] for k = i:min(L,i+l[s])-1 for l = j:min(W,j+w[s])-1))
            end
        end
    end
end

#5 If a slat is used and its BLC occupy a certain position of the panel, the slat can only occupy the following "dimension of the slat positions of the panel
for s = 1:length(S)
    @constraint(model, sum(Z[p,s,i,j] for p = 1:P for i = 0:I for j = 0:J) <= sum(X[p,s,i,j]*min(l[s],L-i)*min(w[s],W-j) for p = 1:P for i = 0:I for j = 0:J))
end

#Solve the model
status = optimize!(model)

#Value of the objective function
nb_panel = objective_value(model)

#Retrieving X, Y and Z
Xsol = value.(X)
Zsol = value.(Z)
Ysol = value.(Y)

#Create the DataFrame in which you would like to store the solution
X_VALUES = DataFrame(Panel = [], Slat = [], Abs = [], Ord = [], Length = [], Width = [])
Y_VALUES = DataFrame(Panel = [])
Z_VALUES = DataFrame(Panel = [], Slat = [], Abs = [], Ord = [])

#Adding the solutions in the DataFrames
#X
for p = 1:P
    for s = 1:length(S)
        for i = 0:I
            for j = 0:J
                val = Xsol[p,s,i,j]
                if val == 1
                    length = l[s]
                    width = w[s]
                    push!(X_VALUES, vcat(p, s, i, j, length, width))
                end
            end
        end
    end
end
X_VALUES

#Y
for p = 1:P
    val = Ysol[p]
    if val == 1
        push!(Y_VALUES, vcat(p))
    end
end
Y_VALUES

#Z
for p = 1:P
    for s = 1:length(S)
        for i = 0:I
            for j = 0:J
                value = Zsol[p,s,i,j]
                if value == 1
                    push!(Z_VALUES, vcat(p, s, i, j))
                end
            end
        end
    end
end
Z_VALUES

#Export the DataFrames into Excel files
XLSX.writetable("MM3_Sol.xlsx", collect(DataFrames.eachcol(X_VALUES)), DataFrames.names(X_VALUES))
d as well
for p = 1:P
    for s = 1:length(S)
        for i = 0:I
          