using DataFrames
using XLSX
using JuMP
using Gurobi
using Gadfly

#data
slats_dim = XLSX.readdata("Dataset0.xlsx", "Slats", "A5:C146")
panels_dim = XLSX.readdata("Dataset0.xlsx", "Panels", "B4:C4")

#Test results
answers = zeros(Int64, 75, 4) #Results vector: nb of lines = 3*nb of tests

#Loop for Tests
nbsl = 15 #nb of slats composing the generated sample
cc = 1 #cc counts the total number of tests
while nbsl < 40 #
    tst = 1 #tst counts the tests until 3 (3 tests for each nb of slats)
    while tst <=3

        #Generation of a random set of slats from real data
        newset = zeros(Int64, nbsl,3)
        for i in 1:nbsl
            rrr = rand(slats_dim[:,1])
            newset[i,1] = i
            newset[i,2] = slats_dim[rrr,2]
            newset[i,3] = slats_dim[rrr,3]
        end

        #Determination of P
        ar = 0
        for i in 1:size(newset,1)
            ar = ar + (newset[i,2]*newset[i,3])
        end

        #Sets
        S = newset[:,1] #Slat associated number
        L = panels_dim[1] #Length of panels to create
        W = panels_dim[2] #Width of panels to create
        P = Int64(floor(ar/(L*W)))
        I = L - 1 #Ordinate of the last gridded area of a panel
        J = W - 1 #Abscissa of the last gridded area of a panel

        #Parameters
        l = newset[:,2] #Length of a slat
        w = newset[:,3] #Width of a slat

        #Model definition
        model = Model(optimizer_with_attributes(Gurobi.Optimizer,"TimeLimit"=>3600))

        #Variables definition
        #X_p,st,s : binary variable, equal to 1 if the Bottom Left Corner (BLC) of slat s is positionned on the position (i,j) of panel p
        @variable(model, X[p = 1:P, s = 1:length(S), i = 0:I, j = 0:J], Bin)
        #Y_p : binary variable, equal to 1 if panel p is created
        @variable(model, Z[p = 1:P, s = 1:length(S), i = 0:I, j = 0:J], Bin)
        #Z_p,st,s : binary variable, equal to 1 if the position (i,j) of panel p is occupied by slat s
        @variable(model, Y[p = 1:P], Bin)

        #Objective function
        @objective(model, Max, sum(Y[p] for p = 1:P))
        #@objective(model, Max, sum(X[p,s,i,j] for p = 1:P for s = 1:length(S) for i = 0:I for j = 0:J))

        #Constraints
        #1 A slay can only be used once
        for s = 1:length(S)
                @constraint(model, sum(X[p,s,i,j] for p = 1:P for i = 0:I for j = 0:J) <= 1)
        end

        #2 All positions of a panel must be occupied in order to create it
        for p = 1:P
            @constraint(model, sum(Z[p,s,i,j] for s = 1:length(S) for i = 0:I for j = 0:J) >= L*W*Y[p])
        end

        #3 A position can only be occupied by one slat (NO OVERLAP)
        for p = 1:P
            for i = 0:I
                for j = 0:J
                    @constraint(model, sum(Z[p,s,i,j] for s = 1:length(S)) <= 1)
                end
            end
        end

        #4 If a position is occupied by the BLC of a slat, the following "dimensions of the slat" positions of the panel are occupied as well
        for p = 1:P
            for s = 1:length(S)
                for i = 0:I
                    for j = 0:J
                        @constraint(model, X[p,s,i,j]*min(l[s],L-i)*min(w[s],W-j) <= sum(Z[p,s,k,l] for k = i:min(L,i+l[s])-1 for l = j:min(W,j+w[s])-1))
                    end
                end
            end
        end

        #5 If a slat is used and its BLC occupy a certain position of the panel, the slat can only occupy the following "dimension of the slat positions of the panel
        for s = 1:length(S)
            @constraint(model, sum(Z[p,s,i,j] for p = 1:P for i = 0:I for j = 0:J) <= sum(X[p,s,i,j]*min(l[s],L-i)*min(w[s],W-j) for p = 1:P for i = 0:I for j = 0:J))
        end

        #Solve the model
        status = optimize!(model)

        #Write answers in vector
        answers[cc,1] = nbsl
        answers[cc,2] = objective_value(model)
        answers[cc,3] = objective_bound(model)
        answers[cc,4] = round(solve_time(model))

        cc = cc + 1 #counts total tests
        tst = tst + 1 #counts the 3 tests loop
    end
    nbsl = nbsl + 1 #go to the following nb of slats
end

answers #results

#Dataframe creation
Test_V = DataFrame(Nb_Slats = [], Best_Sol = [], Best_Bound = [], Computing_Time = [])
for i = 1:size(answers,1)
    push!(Test_V, vcat(answers[i,1], answers[i,2], answers[i,3], answers[i,4]))
end
Test_V

#Write in an Excel file
XLSX.writetable("MM3_Tests_15-39.xlsx", collect(DataFrames.eachcol(Test_V)), DataFrames.names(Test_V))
p,s,i,j] for s = 1:length(S) for i = 0:I for j = 0:J) >= L*W*Y[p])
        end

        #3 A position can only be occupied by one slat (NO OVERLAP)
        for p = 1:P
            for i = 0:I
                for j = 0:J
                    @constraint(model, sum(Z[p,s,i,j] for s = 1:length(S)) <= 1)
                end
            end
        end

        #4 If a position is occupied by the BLC of a slat, the following "dimensions of the slat" positions of the panel are occupied as well
        for p = 1:P
            for s = 1:length(S)
                for i = 0:I
                    for j = 0:J
                        @constraint(model, X[p,s,i,j]*min(l[s],L-i)*min(w[s],W-j) <= sum(Z[p,s,k,l] for k = i:min(L,i+l[s])-1 for l = j:min(W,j+w[s])-1))
                    end
                end
            end
        end

        #5 If a slat is used and its BLC occupy a certain position of the panel, the slat can only occupy the following "dimension of the slat positions of the panel
        for s = 1:length(S)
            @constraint(model, sum(Z[p,s,i,j] for p = 1:P for i = 0:I for j = 0:J) <= sum(X[p,s,i,j]*min(l[s],L-i)*min(w[s],W-j) for p = 1:P for i = 0:I for j = 0:J))
        end

        #Solve the model
        status = optimize!(model)

        #Write answers in vector
        answers[cc,1] = nbsl
        answers[cc,2] = objective_value(model)
        answers[cc,3] = objective_bound(model)
        answers[cc,4] = round(solv